@@ -1,111 +1,7 @@
-#### "TsparseMatrix" : Virtual class of sparse matrices in triplet-format
+## METHODS FOR CLASS: CTsparseMatrix (virtual)
+## sparse matrices in triplet format
+## ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-## more efficient than going via Csparse:
-setAs("matrix", "TsparseMatrix",
-      function(from)
-      if(is.numeric(from)) mat2dgT(from)
-      else if(is.logical(from)) as(Matrix(from, sparse=TRUE), "TsparseMatrix")
-      else stop("not-yet-implemented coercion to \"TsparseMatrix\""))
-
-setAs("numeric", "TsparseMatrix",
-      function(from) as(as.matrix(from), "TsparseMatrix"))
-
-setAs("TsparseMatrix", "matrix",
-      function(from) .Call(dgTMatrix_to_matrix, as(from, "dgTMatrix")))
-
-## in ../src/Tsparse.c :  |-> cholmod_T -> cholmod_C -> chm_sparse_to_SEXP
-## adjusted for triangular matrices not represented in cholmod
-.T.2.C <- function(from) .Call(Tsparse_to_Csparse, from, ##
-			       is(from, "triangularMatrix"))
-## fast, exported for power users
-.T2Cmat <- function(from, isTri = is(from, "triangularMatrix"))
-    .Call(Tsparse_to_Csparse, from, isTri)
-
-
-setAs("TsparseMatrix", "CsparseMatrix", .T.2.C)
-
-.T.2.n <- function(from) {
-    ## No: coercing to n(sparse)Matrix gives the "full" pattern including 0's
-    ## if(any(is0(from@x))) ## 0 or FALSE -- the following should have drop0Tsp(.)
-    ##	from <- as(drop0(from), "TsparseMatrix")
-    if(is(from, "triangularMatrix")) # i.e. ?tTMatrix
-	new("ntTMatrix", i = from@i, j = from@j,
-	    uplo = from@uplo, diag = from@diag,
-	    Dim = from@Dim, Dimnames = from@Dimnames)
-    else if(is(from, "symmetricMatrix")) # i.e. ?sTMatrix
-	new("nsTMatrix", i = from@i, j = from@j, uplo = from@uplo,
-	    Dim = from@Dim, Dimnames = from@Dimnames)
-    else
-	new("ngTMatrix", i = from@i, j = from@j,
-	    Dim = from@Dim, Dimnames = from@Dimnames)
-}
-
-setAs("TsparseMatrix", "nsparseMatrix", .T.2.n)
-setAs("TsparseMatrix", "nMatrix", .T.2.n)
-
-.T.2.l <- function(from) {
-    cld <- getClassDef(class(from))
-    xx <- if(extends(cld, "nMatrix"))
-	rep.int(TRUE, length(from@i)) else as.logical(from@x)
-    if(extends(cld, "triangularMatrix")) # i.e. ?tTMatrix
-	new("ltTMatrix", i = from@i, j = from@j, x = xx,
-	    uplo = from@uplo, diag = from@diag,
-	    Dim = from@Dim, Dimnames = from@Dimnames)
-    else if(extends(cld, "symmetricMatrix")) # i.e. ?sTMatrix
-	new("lsTMatrix", i = from@i, j = from@j, x = xx, uplo = from@uplo,
-	    Dim = from@Dim, Dimnames = from@Dimnames)
-    else
-	new("lgTMatrix", i = from@i, j = from@j, x = xx,
-	    Dim = from@Dim, Dimnames = from@Dimnames)
-}
-
-setAs("TsparseMatrix", "lsparseMatrix", .T.2.l)
-setAs("TsparseMatrix", "lMatrix", .T.2.l)
-
-
-
-## Special cases   ("d", "l", "n")  %o%  ("g", "s", "t") :
-## used e.g. in triu()
-
-setAs("dgTMatrix", "dgCMatrix",
-      function(from) .Call(Tsparse_to_Csparse, from, FALSE))
-
-setAs("dsTMatrix", "dsCMatrix",
-      function(from) .Call(Tsparse_to_Csparse, from, FALSE))
-
-setAs("dtTMatrix", "dtCMatrix",
-      function(from) .Call(Tsparse_to_Csparse, from, TRUE))
-
-
-setAs("lgTMatrix", "lgCMatrix",
-      function(from) .Call(Tsparse_to_Csparse, from, FALSE))
-
-setAs("lsTMatrix", "lsCMatrix",
-      function(from) .Call(Tsparse_to_Csparse, from, FALSE))
-
-setAs("ltTMatrix", "ltCMatrix",
-      function(from) .Call(Tsparse_to_Csparse, from, TRUE))
-
-
-setAs("ngTMatrix", "ngCMatrix",
-      function(from) .Call(Tsparse_to_Csparse, from, FALSE))
-
-setAs("nsTMatrix", "nsCMatrix",
-      function(from) .Call(Tsparse_to_Csparse, from, FALSE))
-
-setAs("ntTMatrix", "ntCMatrix",
-      function(from) .Call(Tsparse_to_Csparse, from, TRUE))
-
-### "[" :
-### -----
-
-## Test for numeric/logical/character
-## method-*internally* ; this is not strictly OO, but allows to use
-## the following utility and hence much more compact code.
-
-## Otherwise have to write methods for all possible combinations of
-##  (i , j) \in
-##  (numeric, logical, character, missing) x (numeric, log., char., miss.)
 
 ##' a simplified "subset" of  intI() below
 int2i <- function(i, n) {
@@ -209,85 +105,6 @@ intI <- function(i, n, dn, give.dn = TRUE)
     intI(i, n = di[margin], dn = dn[[margin]], give.dn = FALSE)
 }
 
-
-## Select rows
-setMethod("[", signature(x = "TsparseMatrix", i = "index", j = "missing",
-			 drop = "logical"),
-	  function (x, i, j, ..., drop) { ## select rows
-	      na <- nargs()
-	      Matrix.msg("Tsp[i,m,l]: nargs()=", na, .M.level=2)
-	      if(na == 4)
-		  .as.Tsp(as(x,"CsparseMatrix")[i, , drop=drop], noCheck = !drop)
-	      else if(na == 3) ## e.g. M[0] , M[TRUE],	M[1:2]
-		  .M.vectorSub(x,i)
-	      else ## should not happen
-		  stop("Matrix-internal error in <TsparseM>[i,,d]; please report")
-	  })
-
-## Select columns
-setMethod("[", signature(x = "TsparseMatrix", i = "missing", j = "index",
-			 drop = "logical"),
-	  function (x, i, j, ..., drop) { ## select columns
-	      .as.Tsp(as(x,"CsparseMatrix")[, j, drop=drop], noCheck = !drop)
-	  })
-
-setMethod("[", signature(x = "TsparseMatrix",
-			 i = "index", j = "index", drop = "logical"),
-	  function (x, i, j, ..., drop)
-	  .as.Tsp(as(x,"CsparseMatrix")[i, j, drop=drop], noCheck = !drop))
-
-## This is "just for now" -- Thinking of *not* doing this in the future
-.as.Tsp <- function(x, noCheck)
-    if(noCheck || is(x,"sparseMatrix")) as(x, "TsparseMatrix") else x
-
-
-## FIXME: Learn from .TM... below or rather  .M.sub.i.2col(.) in ./Matrix.R
-## ------ the following should be much more efficient than the
-##  subset.ij() based ./Matrix.R code :
-if(FALSE)
-## A[ ij ]  where ij is (i,j) 2-column matrix :
-setMethod("[", signature(x = "TsparseMatrix",
-			 i = "matrix", j = "missing"),# drop="ANY"
-	  function (x, i, j, ..., drop)
-      {
-	  di <- dim(x)
-	  dn <- dimnames(x)
-	  ## TODO check	 i (= 2-column matrix of indices) ---
-	  ##	  as in	 .M.sub.i.2col() in ./Matrix.R
-	  j <- i[,2]
-	  i <- i[,1]
-	  if(is(x, "symmetricMatrix")) {
-	      isSym <- isTRUE(all(i == j))# work for i,j NA
-	      if(!isSym)
-		  x <- as(x, paste0(.M.kind(x), "gTMatrix"))
-	  } else isSym <- FALSE
-
-	  if(isSym) {
-	      offD <- x@i != x@j
-	      ip1 <- .ind.prep(c(x@i,x@j[offD]), intI(i, n= di[1], dn=dn[[1]]))
-	      ip2 <- .ind.prep(c(x@j,x@i[offD]), intI(j, n= di[2], dn=dn[[2]]))
-	  } else {
-	      ip1 <- .ind.prep(x@i, intI(i, n = di[1], dn = dn[[1]]))
-	      ip2 <- .ind.prep(x@j, intI(j, n = di[2], dn = dn[[2]]))
-	  }
-
-	  stop("FIXME: NOT YET FINISHED IMPLEMENTATION")
-
-	  ## The M[i_vec, j_vec] had -- we need "its diagonal" :
-	  sel <- ip1$m	&  ip2$m
-	  if(isSym) { # only those corresponding to upper/lower triangle
-	      sel <- sel &
-	      (if(x@uplo == "U") ip1$m <= ip2$m else ip2$m <= ip1$m)
-	  }
-	  x@i <- ip1$m[sel] - 1L
-	  x@j <- ip2$m[sel] - 1L
-	  if (!is(x, "nsparseMatrix"))
-	      x@x <- c(x@x, if(isSym) x@x[offD])[sel]
-	  if (drop && any(nd == 1)) drop(as(x,"matrix")) else x
-
-      })
-
-
 ###========= Sub-Assignment aka *Replace*Methods =========================
 
 ### FIXME: make this `very fast'  for the very very common case of
@@ -334,7 +151,7 @@ replTmat <- function (x, i, j, ..., value)
 
 	if(!is(x,"generalMatrix")) {
 	    cl <- class(x)
-	    x <- as(x, paste0(.M.kind(x), "gTMatrix"))
+	    x <- .M2gen(x)
 	    Matrix.msg("'sub-optimal sparse 'x[i] <- v' assignment: Coercing class ",
 		       cl," to ",class(x))
 	}
@@ -386,7 +203,7 @@ replTmat <- function (x, i, j, ..., value)
 		x@j <- x@j[sel]
 		if(has.x)
 		    x@x <- x@x[sel]
-		if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
+		if(.hasSlot(x, "factors") && length(x@factors)) # drop cached ones
 		    x@factors <- list()
 	    }
 	    return(x)
@@ -439,7 +256,7 @@ replTmat <- function (x, i, j, ..., value)
 	    x@i <- c(x@i, i %%  nr)
 	    x@j <- c(x@j, i %/% nr)
 	}
-	if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
+	if(.hasSlot(x, "factors") && length(x@factors)) # drop cached ones
 	    x@factors <- list()
 	return(x)
     } ## {nargs = 3;  x[ii] <- value }
@@ -524,11 +341,8 @@ replTmat <- function (x, i, j, ..., value)
 	else toGeneral <- TRUE
     }
     if(toGeneral) { # go to "generalMatrix" and continue
-	if((.w <- isTRUE(getOption("Matrix.warn"))) ||
-	   (!is.null(v <- getOption("Matrix.verbose")) && v >= 1))
-	    (if(.w) warning else message)(
-	     "M[i,j] <- v :  coercing symmetric M[] into non-symmetric")
-        x <- as(x, paste0(.M.kind(x), "gTMatrix"))
+        Matrix.msg("M[i,j] <- v :  coercing symmetric M[] into non-symmetric")
+        x <- .M2gen(x)
         clDx <- getClassDef(clx <- class(x))
     }
 
@@ -547,7 +361,7 @@ replTmat <- function (x, i, j, ..., value)
 	    x@j <- x@j[!sel]
             if(has.x)
 		x@x <- x@x[!sel]
-	    if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
+	    if(.hasSlot(x, "factors") && length(x@factors)) # drop cached ones
 		x@factors <- list()
 	}
 	return(x)
@@ -578,7 +392,7 @@ replTmat <- function (x, i, j, ..., value)
             if(has.x)
                 x@x <- c(x@x, value)
         }
-	if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
+	if(.hasSlot(x, "factors") && length(x@factors)) # drop cached ones
 	    x@factors <- list()
         return(x)
     }
@@ -589,10 +403,10 @@ replTmat <- function (x, i, j, ..., value)
 ## FIXME: just for testing !!
 ## if(identical(Sys.getenv("USER"),"maechler")
 ##    if(lenRepl > 2) { # __________ ___ JUST for testing! _______________
-	if(nonTRUEoption("Matrix.quiet"))
+	if(!isTRUE(getOption("Matrix.quiet")))
 	    message(gettextf("x[.,.] <- val : x being coerced from Tsparse* to CsparseMatrix"),
 		    domain = NA)
-	return(replCmat4(as(x,"CsparseMatrix"), i1, i2, iMi=iMi, jMi=jMi,
+	return(replCmat4(.M2C(x), i1, i2, iMi=iMi, jMi=jMi,
 			 value = if(spV) value else as(value, "sparseVector"),
 			 spV = TRUE))
     }
@@ -675,7 +489,7 @@ replTmat <- function (x, i, j, ..., value)
 		x@x <- c(x@x, as.vector(value[iIN0]))
 	}
     }
-    if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
+    if(.hasSlot(x, "factors") && length(x@factors)) # drop cached ones
 	x@factors <- list()
     x
 } ## end{replTmat}
@@ -785,17 +599,14 @@ replTmat <- function (x, i, j, ..., value)
 	else toGeneral <- TRUE
     }
     if(toGeneral) { # go to "generalMatrix" and continue
-	if((.w <- isTRUE(getOption("Matrix.warn"))) || isTRUE(getOption("Matrix.verbose")))
-	    (if(.w) warning else message)(
-	     "M[ij] <- v :  coercing symmetric M[] into non-symmetric")
-	x <- as(x, paste0(.M.kind(x), "gTMatrix"))
+	Matrix.msg("M[ij] <- v :  coercing symmetric M[] into non-symmetric")
+	x <- .M2gen(x)
 	clDx <- getClassDef(clx <- class(x))
     }
 
     ii.v <- .Call(m_encodeInd, i, di, orig1=TRUE, checkBounds = TRUE)
     if(id <- anyDuplicated(ii.v, fromLast=TRUE)) {
-        Matrix.msg("duplicate ij-entries in 'Matrix[ ij ] <- value'; using last",
-                   .M.level = 1)
+        Matrix.msg("M[ij] <- v :  duplicate ij-entries; using last")
         ii.v  <- ii.v [-id]
 	value <- value[-id]
         if(any(id <- duplicated(ii.v, fromLast=TRUE))) {
@@ -809,10 +620,9 @@ replTmat <- function (x, i, j, ..., value)
     i.repl <- !is.na(m1) # those that need to be *replaced*
 
     if(isN) { ## no 'x' slot
-	isN <- all(value %in% c(FALSE, TRUE)) # will result remain  "nMatrix" ?
+	isN <- is.logical(value) # will result remain  "nMatrix" ?
 	if(!isN)
-	    x <- as(x, paste0(if(extends(clDx, "lMatrix")) "l" else "d",
-			      .sparse.prefixes[.M.shape(x)], "TMatrix"))
+            x <- .M2kind(x, "d")
     }
     has.x <- !isN ## isN  <===> "remains pattern matrix" <===> has no 'x' slot
 
@@ -820,7 +630,7 @@ replTmat <- function (x, i, j, ..., value)
 	if(has.x)
 	    x@x[m1[i.repl]] <- value[i.repl]
 	else { # nMatrix ; eliminate entries that are set to FALSE; keep others
-	    if(any(isF <- !value[i.repl]))  {
+	    if(any(isF <- is0(value[i.repl])))  {
 		ii <- m1[i.repl][isF]
 		x@i <- x@i[ -ii]
 		x@j <- x@j[ -ii]
@@ -835,7 +645,7 @@ replTmat <- function (x, i, j, ..., value)
 	    x@x <- c(x@x, value[i.new])
     }
 
-    if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
+    if(.hasSlot(x, "factors") && length(x@factors)) # drop cached ones
 	x@factors <- list()
     x
 } ## end{.TM.repl.i.mat}
@@ -872,48 +682,3 @@ setReplaceMethod("[", signature(x = "TsparseMatrix", i = "index", j = "missing",
 setReplaceMethod("[", signature(x = "TsparseMatrix", i = "index", j = "index",
 				value = "sparseVector"),
 		 replTmat)
-
-
-
-
-setMethod("solve", signature(a = "TsparseMatrix", b = "ANY"),
-	  function(a, b, ...) solve(as(a, "CsparseMatrix"), b))
-setMethod("solve", signature(a = "TsparseMatrix", b = "missing"),
-	  function(a, b, ...) solve(as(a, "CsparseMatrix")))
-
-
-## Want tril(), triu(), band() --- just as "indexing" ---
-## return a "close" class:
-setMethod("tril", "TsparseMatrix",
-	  function(x, k = 0, ...)
-	  as(tril(.T.2.C(x), k = k, ...), "TsparseMatrix"))
-setMethod("triu", "TsparseMatrix",
-	  function(x, k = 0, ...)
-	  as(triu(.T.2.C(x), k = k, ...), "TsparseMatrix"))
-setMethod("band", "TsparseMatrix",
-	  function(x, k1, k2, ...)
-	  as(band(.T.2.C(x), k1 = k1, k2 = k2, ...), "TsparseMatrix"))
-
-
-## For the "general" T ones (triangular & symmetric have special methods):
-setMethod("t", signature(x = "TsparseMatrix"),
-	  function(x) {
-              cld <- getClassDef(class(x))
-	      r <- new(cld)
-	      r@i <- x@j
-	      r@j <- x@i
-	      if(any("x" == slotNames(cld)))
-		  r@x <- x@x
-	      r@Dim <- x@Dim[2:1]
-	      r@Dimnames <- x@Dimnames[2:1]
-	      r
-      })
-
-isDiagTsp <- function(object) {
-    d <- dim(object)
-    if(d[1] != d[2])
-        FALSE
-    else
-        length(i <- object@i) == length(j <- object@j) && all(i == j)
-}
-setMethod("isDiagonal", signature(object = "TsparseMatrix"), isDiagTsp)
